'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _mapEvents = require('./utilities/mapEvents');

var _mapEvents2 = _interopRequireDefault(_mapEvents);

var _requestPromise = require('request-promise');

var _requestPromise2 = _interopRequireDefault(_requestPromise);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)('geteventstore:getAllStreamEvents');
var baseErr = 'Get All Stream Events - ';

var buildOptions = function buildOptions(config, streamName, startPosition, chunkSize, resolveLinkTos, embed) {
	return {
		uri: config.baseUrl + '/streams/' + streamName + '/' + startPosition + '/forward/' + chunkSize,
		method: 'GET',
		headers: {
			"Content-Type": "application/vnd.eventstore.events+json",
			"ES-ResolveLinkTos": resolveLinkTos.toString()
		},
		qs: {
			embed: embed
		},
		json: true,
		timeout: config.timeout
	};
};

exports.default = function (config) {
	return function () {
		var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(streamName, chunkSize, startPosition, resolveLinkTos) {
			var embed = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 'body';
			var events, getNextChunk;
			return _regenerator2.default.wrap(function _callee2$(_context2) {
				while (1) {
					switch (_context2.prev = _context2.next) {
						case 0:
							(0, _assert2.default)(streamName, baseErr + 'Stream Name not provided');

							startPosition = startPosition || 0;
							chunkSize = chunkSize || 1000;
							resolveLinkTos = resolveLinkTos === undefined ? true : resolveLinkTos;

							if (chunkSize > 4096) {
								console.warn('WARNING: Max event chunk size exceeded. Using the max of 4096');
								chunkSize = 4096;
							}

							events = [];

							getNextChunk = function () {
								var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
									var options, response, totalEntries, i;
									return _regenerator2.default.wrap(function _callee$(_context) {
										while (1) {
											switch (_context.prev = _context.next) {
												case 0:
													options = buildOptions(config, streamName, startPosition, chunkSize, resolveLinkTos, embed);
													_context.next = 3;
													return (0, _requestPromise2.default)(options);

												case 3:
													response = _context.sent;

													debug('', 'Result: %j', response);

													if (embed === 'body') {
														totalEntries = response.entries.length;

														for (i = 0; i < totalEntries; i++) {
															if (response.entries[i].data) response.entries[i].data = JSON.parse(response.entries[i].data);
														}
													}

													events.push(response.entries.reverse());

													if (!(response.headOfStream === true)) {
														_context.next = 9;
														break;
													}

													return _context.abrupt('return', (0, _mapEvents2.default)(_lodash2.default.flatten(events)));

												case 9:

													startPosition += chunkSize;
													return _context.abrupt('return', getNextChunk());

												case 11:
												case 'end':
													return _context.stop();
											}
										}
									}, _callee, undefined);
								}));

								return function getNextChunk() {
									return _ref2.apply(this, arguments);
								};
							}();

							return _context2.abrupt('return', getNextChunk());

						case 8:
						case 'end':
							return _context2.stop();
					}
				}
			}, _callee2, undefined);
		}));

		return function (_x2, _x3, _x4, _x5) {
			return _ref.apply(this, arguments);
		};
	}();
};

module.exports = exports['default'];