'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var baseErr = 'geteventstore-promise - HTTP client - ';

exports.default = function (config) {
	//Assert configuration
	(0, _assert2.default)(config, baseErr + 'config not provided');
	(0, _assert2.default)(config.hostname, baseErr + 'hostname property not provided');
	(0, _assert2.default)(config.port, baseErr + 'port property not provided');
	(0, _assert2.default)(config.credentials, baseErr + 'credentials property not provided');
	(0, _assert2.default)(config.credentials.username, baseErr + 'credentials.username property not provided');
	(0, _assert2.default)(config.credentials.password, baseErr + 'credentials.password property not provided');

	if (config.timeout) (0, _assert2.default)(_lodash2.default.isNumber(config.timeout), baseErr + 'timeout not defined');
	//Add additional internal configuration properties
	config = JSON.parse((0, _stringify2.default)(config));
	config.protocol = 'http';
	config.auth = config.credentials.username + ':' + config.credentials.password;
	config.baseUrl = _url2.default.format(config);

	return {
		checkStreamExists: require('./checkStreamExists')(config),
		writeEvent: require('./writeEvent')(config),
		writeEvents: require('./writeEvents')(config),
		getAllStreamEvents: require('./getAllStreamEvents')(config),
		getEvents: require('./getEvents')(config),
		deleteStream: require('./deleteStream')(config),
		ping: require('./ping')(config),
		admin: {
			scavenge: require('./admin/sendScavengeCommand')(config),
			shutdown: require('./admin/sendShutdownCommand')(config)
		},
		projections: {
			start: require('./projections/start')(config),
			stop: require('./projections/stop')(config),
			reset: require('./projections/reset')(config),
			assert: require('./projections/assert')(config),
			remove: require('./projections/remove')(config),
			getState: require('./projections/getState')(config),
			getInfo: require('./projections/getInfo')(config),
			getAllProjectionsInfo: require('./projections/getAllProjectionsInfo')(config),
			disableAll: require('./projections/disableAll')(config),
			enableAll: require('./projections/enableAll')(config)
		},
		persistentSubscriptions: {
			assert: require('./persistentSubscriptions/assert')(config),
			remove: require('./persistentSubscriptions/remove')(config),
			getEvents: require('./persistentSubscriptions/getEvents')(config),
			getSubscriptionInfo: require('./persistentSubscriptions/getSubscriptionInfo')(config),
			getAllSubscriptionsInfo: require('./persistentSubscriptions/getAllSubscriptionsInfo')(config),
			getStreamSubscriptionsInfo: require('./persistentSubscriptions/getStreamSubscriptionsInfo')(config)
		}
	};
};

module.exports = exports['default'];