'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _requestPromise = require('request-promise');

var _requestPromise2 = _interopRequireDefault(_requestPromise);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)('geteventstore:assertPersistentSubscription');
var baseErr = 'Assert persistent subscriptions - ';

var createPersistentSubscriptionRequest = function createPersistentSubscriptionRequest(config, name, streamName, options) {
	return {
		uri: config.baseUrl + '/subscriptions/' + streamName + '/' + name,
		method: 'PUT',
		json: true,
		headers: {
			'Content-Type': 'application/json'
		},
		body: options
	};
};

var createPersistentSubscriptionOptions = function createPersistentSubscriptionOptions(options) {
	options = options || {};

	return {
		resolveLinktos: options.resolveLinktos,
		startFrom: options.startFrom === undefined ? 0 : options.startFrom,
		extraStatistics: options.extraStatistics,
		checkPointAfterMilliseconds: options.checkPointAfterMilliseconds,
		liveBufferSize: options.liveBufferSize,
		readBatchSize: options.readBatchSize,
		bufferSize: options.bufferSize,
		maxCheckPointCount: options.maxCheckPointCount,
		maxRetryCount: options.maxRetryCount,
		maxSubscriberCount: options.maxSubscriberCount,
		messageTimeoutMilliseconds: options.messageTimeoutMilliseconds,
		minCheckPointCount: options.minCheckPointCount,
		namedConsumerStrategy: options.namedConsumerStrategy
	};
};

exports.default = function (config) {
	return function () {
		var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(name, streamName, options) {
			var persistentSubscriptionOptions, createRequest, response, updateRequest, _response;

			return _regenerator2.default.wrap(function _callee$(_context) {
				while (1) {
					switch (_context.prev = _context.next) {
						case 0:
							(0, _assert2.default)(name, baseErr + 'Persistent Subscription Name not provided');
							(0, _assert2.default)(streamName, baseErr + 'Stream Name not provided');

							persistentSubscriptionOptions = createPersistentSubscriptionOptions(options);
							createRequest = createPersistentSubscriptionRequest(config, name, streamName, persistentSubscriptionOptions);
							_context.prev = 4;

							debug('', 'Options: %j', createRequest);
							_context.next = 8;
							return (0, _requestPromise2.default)(createRequest);

						case 8:
							response = _context.sent;

							debug('', 'Response: %j', response);
							return _context.abrupt('return', response);

						case 13:
							_context.prev = 13;
							_context.t0 = _context['catch'](4);

							if (!(_context.t0.statusCode !== 409)) {
								_context.next = 17;
								break;
							}

							throw _context.t0;

						case 17:
							updateRequest = createPersistentSubscriptionRequest(config, name, streamName, persistentSubscriptionOptions);

							updateRequest.method = 'POST';

							debug('', 'Update Options: %j', updateRequest);
							_context.next = 22;
							return (0, _requestPromise2.default)(updateRequest);

						case 22:
							_response = _context.sent;

							debug('', 'Response: %j', _response);
							return _context.abrupt('return', _response);

						case 25:
						case 'end':
							return _context.stop();
					}
				}
			}, _callee, undefined, [[4, 13]]);
		}));

		return function (_x, _x2, _x3) {
			return _ref.apply(this, arguments);
		};
	}();
};

module.exports = exports['default'];