'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _requestPromise = require('request-promise');

var _requestPromise2 = _interopRequireDefault(_requestPromise);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)('geteventstore:getEventsPersistentSubscription');
var baseErr = 'Get persistent subscriptions events - ';

var createRequest = function createRequest(config, name, streamName, count, embed) {
	var request = {
		uri: config.baseUrl + '/subscriptions/' + streamName + '/' + name + '/' + count + '?embed=' + embed,
		method: 'GET',
		json: true,
		headers: {
			'Accept': 'application/vnd.eventstore.competingatom+json'
		}
	};
	return request;
};

var postUrl = function postUrl(uri) {
	return function () {
		var postRequest = {
			uri: uri,
			method: 'POST',
			json: true,
			headers: {
				'Content-Type': 'application/json'
			}
		};
		return (0, _requestPromise2.default)(postRequest);
	};
};

var appendLinkFunctions = function appendLinkFunctions(resultObject, links) {
	_lodash2.default.each(links, function (link) {
		return resultObject[link.relation] = postUrl(link.uri);
	});
};

var buildResultObject = function buildResultObject(response) {
	debug('', 'Response: %j', response);
	var result = {
		entries: []
	};

	appendLinkFunctions(result, response.links);

	result.entries = _lodash2.default.map(response.entries, function (entry) {
		if (entry.data) entry.data = JSON.parse(entry.data);

		var formattedEntry = {
			event: entry
		};
		appendLinkFunctions(formattedEntry, entry.links);
		return formattedEntry;
	});

	debug('', 'Result: %j', result);
	return result;
};

exports.default = function (config) {
	return function () {
		var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(name, streamName, count, embed) {
			var options;
			return _regenerator2.default.wrap(function _callee$(_context) {
				while (1) {
					switch (_context.prev = _context.next) {
						case 0:
							(0, _assert2.default)(name, baseErr + 'Persistent Subscription Name not provided');
							(0, _assert2.default)(streamName, baseErr + 'Stream Name not provided');

							count = count === undefined ? 1 : count;
							embed = embed || 'Body';

							options = createRequest(config, name, streamName, count, embed);

							debug('', 'Options: %j', options);
							return _context.abrupt('return', (0, _requestPromise2.default)(options).then(buildResultObject));

						case 7:
						case 'end':
							return _context.stop();
					}
				}
			}, _callee, undefined);
		}));

		return function (_x, _x2, _x3, _x4) {
			return _ref.apply(this, arguments);
		};
	}();
};

module.exports = exports['default'];