'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _nodeEventstoreClient = require('node-eventstore-client');

var _nodeEventstoreClient2 = _interopRequireDefault(_nodeEventstoreClient);

var _genericPool = require('generic-pool');

var _genericPool2 = _interopRequireDefault(_genericPool);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _lodash = require('lodash');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)('geteventstore:connectionManager');
var _uniqueConfigConnectionPools = [];

exports.default = {
	create: function () {
		var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(config, onConnected) {
			var isSubscription = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
			var connectionPool, opts, factory;
			return _regenerator2.default.wrap(function _callee2$(_context2) {
				while (1) {
					switch (_context2.prev = _context2.next) {
						case 0:
							connectionPool = (0, _lodash.find)(_uniqueConfigConnectionPools, { config: config, onConnected: onConnected });


							if (!connectionPool) {
								opts = config.poolOptions || { autostart: false };

								if (isSubscription) {
									opts.min = 1;
									opts.max = 1;
								}

								factory = {
									create: function () {
										var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
											var client;
											return _regenerator2.default.wrap(function _callee$(_context) {
												while (1) {
													switch (_context.prev = _context.next) {
														case 0:
															_context.next = 2;
															return _nodeEventstoreClient2.default.EventStoreConnection.create(config, config.gossipSeeds || config.protocol + '://' + config.hostname + ':' + config.port);

														case 2:
															client = _context.sent;


															client.releaseConnection = function () {
																return connectionPool.pool.release(client);
															};
															if (onConnected) {
																client.once('connected', onConnected);
															} else {
																client.on('connected', function () {
																	return debug('', client._connectionName + ' - Connection Connected');
																});
															}
															client.on('disconnected', function () {
																return debug('', client._connectionName + ' - Connection Disconnected');
															});
															client.on('reconnecting', function () {
																return debug('', client._connectionName + ' - Connection Reconnecting...');
															});
															client.on('closed', function (reason) {
																debug('', client._connectionName + ' - Connection Closed: ' + reason);
																factory.destroy(client);
															});
															client.on('error', function (err) {
																debug('', client._connectionName + ' - Connection Error: ' + err.stack);
																console.error(client._connectionName + ' - EventStore: ' + err.stack);
																try {
																	factory.destroy(client);
																} catch (ex) {
																	//Do nothing
																}
															});

															_context.next = 11;
															return client.connect();

														case 11:
															return _context.abrupt('return', client);

														case 12:
														case 'end':
															return _context.stop();
													}
												}
											}, _callee, this);
										}));

										function create() {
											return _ref2.apply(this, arguments);
										}

										return create;
									}(),
									destroy: function destroy(client) {
										return client.close();
									}
								};


								connectionPool = {
									config: config,
									onConnected: onConnected,
									pool: _genericPool2.default.createPool(factory, opts)
								};
								_uniqueConfigConnectionPools.push(connectionPool);
							}

							return _context2.abrupt('return', connectionPool.pool.acquire());

						case 3:
						case 'end':
							return _context2.stop();
					}
				}
			}, _callee2, this);
		}));

		function create(_x2, _x3) {
			return _ref.apply(this, arguments);
		}

		return create;
	}(),
	closeAllPools: function () {
		var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
			return _regenerator2.default.wrap(function _callee3$(_context3) {
				while (1) {
					switch (_context3.prev = _context3.next) {
						case 0:
							_context3.next = 2;
							return _bluebird2.default.all(_uniqueConfigConnectionPools.map(function (connectionPool) {
								return connectionPool.pool.clear();
							}));

						case 2:
							_uniqueConfigConnectionPools.splice(0, _uniqueConfigConnectionPools.length);

						case 3:
						case 'end':
							return _context3.stop();
					}
				}
			}, _callee3, this);
		}));

		function closeAllPools() {
			return _ref3.apply(this, arguments);
		}

		return closeAllPools;
	}(),
	close: function close(config) {
		var _this = this;

		return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4() {
			var pool;
			return _regenerator2.default.wrap(function _callee4$(_context4) {
				while (1) {
					switch (_context4.prev = _context4.next) {
						case 0:
							_context4.next = 2;
							return _this.getPool(config)();

						case 2:
							pool = _context4.sent;
							_context4.next = 5;
							return pool.clear();

						case 5:
							_uniqueConfigConnectionPools = _uniqueConfigConnectionPools.filter(function (_connectionPool) {
								return _connectionPool.pool !== pool;
							});

						case 6:
						case 'end':
							return _context4.stop();
					}
				}
			}, _callee4, _this);
		}));
	},
	getPool: function getPool(config) {
		var _this2 = this;

		return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5() {
			var connectionPool;
			return _regenerator2.default.wrap(function _callee5$(_context5) {
				while (1) {
					switch (_context5.prev = _context5.next) {
						case 0:
							connectionPool = (0, _lodash.find)(_uniqueConfigConnectionPools, { config: config });

							if (connectionPool) {
								_context5.next = 3;
								break;
							}

							throw new Error('Connection Pool not found');

						case 3:
							return _context5.abrupt('return', connectionPool.pool);

						case 4:
						case 'end':
							return _context5.stop();
					}
				}
			}, _callee5, _this2);
		}));
	}
};
module.exports = exports['default'];