'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _connectionManager = require('./connectionManager');

var _connectionManager2 = _interopRequireDefault(_connectionManager);

var _mapEvents = require('./utilities/mapEvents');

var _mapEvents2 = _interopRequireDefault(_mapEvents);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)('geteventstore:eventEnumerator');
var baseErr = 'Event Enumerator - ';

var getNextBatch = function () {
	var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(config, streamName, state, length, direction, resolveLinkTos) {
		var connection, handleResult;
		return _regenerator2.default.wrap(function _callee$(_context) {
			while (1) {
				switch (_context.prev = _context.next) {
					case 0:
						handleResult = function handleResult(result) {
							debug('', 'Result: %j', result);

							state.nextEventNumber = result.nextEventNumber.toNumber ? result.nextEventNumber.toNumber() : result.nextEventNumber;
							return {
								isEndOfStream: result.isEndOfStream,
								events: (0, _mapEvents2.default)(result.events)
							};
						};

						state.isFirstEnumeration = false;
						(0, _assert2.default)(streamName, baseErr + 'Stream Name not provided');

						_context.next = 5;
						return _connectionManager2.default.create(config);

					case 5:
						connection = _context.sent;
						_context.prev = 6;

						if (!(direction === 'forward')) {
							_context.next = 11;
							break;
						}

						_context.next = 10;
						return connection.readStreamEventsForward(streamName, state.nextEventNumber, length, resolveLinkTos, config.credentials).then(handleResult);

					case 10:
						return _context.abrupt('return', _context.sent);

					case 11:
						_context.next = 13;
						return connection.readStreamEventsBackward(streamName, state.nextEventNumber, length, resolveLinkTos, config.credentials).then(handleResult);

					case 13:
						return _context.abrupt('return', _context.sent);

					case 16:
						_context.prev = 16;
						_context.t0 = _context['catch'](6);
						throw _context.t0;

					case 19:
						_context.prev = 19;

						connection.releaseConnection();
						return _context.finish(19);

					case 22:
					case 'end':
						return _context.stop();
				}
			}
		}, _callee, undefined, [[6, 16, 19, 22]]);
	}));

	return function getNextBatch(_x, _x2, _x3, _x4, _x5, _x6) {
		return _ref.apply(this, arguments);
	};
}();

var esDirectionWorkaroundHandler = function esDirectionWorkaroundHandler(direction) {
	var wasSwopped = false;

	if (direction === 'forward') {
		wasSwopped = true;
		direction = 'backward';
	}

	return {
		direction: direction,
		swopResult: function swopResult(state, length, result) {
			if (wasSwopped) {
				state.nextEventNumber += length + 1;
				result.events.reverse();
			}
			return result;
		}
	};
};

var stateHandler = function stateHandler(direction) {
	var Handler = function () {
		function Handler() {
			(0, _classCallCheck3.default)(this, Handler);

			this.isFirstEnumeration = true;
			this.setToFirst();
		}

		(0, _createClass3.default)(Handler, [{
			key: 'setToFirst',
			value: function setToFirst() {
				this.nextEventNumber = direction === 'forward' ? 0 : -1;
			}
		}, {
			key: 'setToLast',
			value: function setToLast(length) {
				this.nextEventNumber = direction === 'forward' ? -1 : length - 1;
			}
		}, {
			key: 'setToPrevious',
			value: function setToPrevious(length) {
				if (!this.isFirstEnumeration) this.adjustByLength(length);
			}
		}, {
			key: 'keepInBoundsAdjustment',
			value: function keepInBoundsAdjustment(length) {
				if (direction === 'backward') return length;

				var adjustment = length;
				if (this.nextEventNumber < -1) {
					adjustment -= Math.abs(this.nextEventNumber);
					this.nextEventNumber = 0;
				}

				return adjustment;
			}
		}, {
			key: 'adjustByLength',
			value: function adjustByLength(length) {
				this.nextEventNumber += direction === 'forward' ? length * -1 : length;
			}
		}]);
		return Handler;
	}();

	return new Handler();
};

exports.default = function (config) {
	return function (streamName, direction, resolveLinkTos) {
		direction = direction || 'forward';
		resolveLinkTos = resolveLinkTos === undefined ? true : resolveLinkTos;
		var state = stateHandler(direction);

		return {
			first: function first(length) {
				state.setToFirst();
				return getNextBatch(config, streamName, state, length, direction, resolveLinkTos);
			},
			last: function last(length) {
				state.setToLast(length);

				var handler = esDirectionWorkaroundHandler(direction);
				return getNextBatch(config, streamName, state, length, handler.direction, resolveLinkTos).then(function (result) {
					return handler.swopResult(state, length, result);
				});
			},
			previous: function previous(length) {
				state.setToPrevious(length);
				length = state.keepInBoundsAdjustment(length);

				return getNextBatch(config, streamName, state, length, direction, resolveLinkTos).then(function (result) {
					state.adjustByLength(length);
					return result;
				});
			},
			next: function next(length) {
				return getNextBatch(config, streamName, state, length, direction, resolveLinkTos);
			}
		};
	};
};

module.exports = exports['default'];