'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _nodeEventstoreClient = require('node-eventstore-client');

var _nodeEventstoreClient2 = _interopRequireDefault(_nodeEventstoreClient);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var baseErr = 'geteventstore-promise - TCP client - ';

exports.default = function (config) {
	//Assert configuration
	(0, _assert2.default)(config, baseErr + 'config not provided');
	if (config.gossipSeeds) {
		(0, _assert2.default)(Array.isArray(config.gossipSeeds), baseErr + 'gossipSeeds must be an array');
		(0, _assert2.default)(config.gossipSeeds.length >= 3, baseErr + 'at least 3 gossipSeeds are required');

		config.gossipSeeds.forEach(function (seed) {
			(0, _assert2.default)(seed.hostname, baseErr + 'gossip seed must have a hostname');
			(0, _assert2.default)(seed.port, baseErr + 'gossip seed must have a port');
		});
	} else {
		(0, _assert2.default)(config.hostname, baseErr + 'hostname property not provided');
		(0, _assert2.default)(config.port, baseErr + 'port property not provided');
	}
	(0, _assert2.default)(config.credentials, baseErr + 'credentials property not provided');
	(0, _assert2.default)(config.credentials.username, baseErr + 'credentials.username property not provided');
	(0, _assert2.default)(config.credentials.password, baseErr + 'credentials.password property not provided');

	//Add additional internal configuration properties
	config = JSON.parse((0, _stringify2.default)(config));
	config.protocol = config.protocol || 'tcp';
	config.host = config.hostname;
	config.auth = config.credentials.username + ':' + config.credentials.password;
	config.baseUrl = _url2.default.format(config);

	if (config.gossipSeeds && config.gossipSeeds.length > 0) {
		config.gossipSeeds = config.gossipSeeds.map(function (seed) {
			return new _nodeEventstoreClient2.default.GossipSeed({ host: seed.hostname, port: seed.port }, seed.hostHeader);
		});
	}

	return {
		checkStreamExists: require('./checkStreamExists')(config),
		writeEvent: require('./writeEvent')(config),
		writeEvents: require('./writeEvents')(config),
		getAllStreamEvents: require('./getAllStreamEvents')(config),
		getEvents: require('./getEvents')(config),
		getEventsByType: require('./getEventsByType')(config),
		deleteStream: require('./deleteStream')(config),
		eventEnumerator: require('./eventEnumerator')(config),
		subscribeToStream: require('./subscribeToStream')(config),
		subscribeToStreamFrom: require('./subscribeToStreamFrom')(config),
		close: require('./connectionManager').close(config),
		getPool: require('./connectionManager').getPool(config),
		closeAllPools: require('./connectionManager').closeAllPools
	};
};

module.exports = exports['default'];